#include <QtGui>
#include <iostream>
#include <fstream>
#include "bscannyitem.h"
#include "pt2i.h"

using namespace std;


const int BSCannyItem::ALONE = 0;
const int BSCannyItem::WITH_BS = 1;
const int BSCannyItem::WITH_DSS = 2;
const QColor BSCannyItem::BS_COLOR = Qt::blue;
const QColor BSCannyItem::CANNY_COLOR = Qt::green;
const QColor BSCannyItem::BOTH_COLOR = Qt::red;
const int BSCannyItem::DEFAULT_PEN_WIDTH = 1;


BSCannyItem::BSCannyItem (int width, int height, const QImage *im,
                          BSDetector *detector)
{
  w = width;
  h = height;
  int size = w * h;
  this->im = im;
  bsmap = new bool[size];
  clmap = new bool[size];
  det = detector;
  type = ALONE;
  int nb = readCannyLines ();
  cout << nb << " cannys" << endl;
}


BSCannyItem::~BSCannyItem ()
{
  delete [] bsmap;
}


QRectF BSCannyItem::boundingRect () const
{
  return QRectF (0, 0, w - 1, h - 1); // ZZZ
}


void BSCannyItem::paint (QPainter *painter,
                         const QStyleOptionGraphicsItem *option,
                         QWidget *widget)
{
  Q_UNUSED (option);
  Q_UNUSED (widget);

  if (type == WITH_BS) drawBS (painter);
  else if (type == WITH_DSS) drawDSS (painter);
  drawCannyLines (painter);
  int nboth = 0, ncl = 0, nbs = 0;
  for (int i = 0; i < w * h; i++)
    if (clmap[i])
      if (bsmap[i]) nboth++;
      else ncl++;
    else if (bsmap[i]) nbs++;
  cout << nboth << " both, " << ncl << " canny, " << nbs << " bs" << endl;
}


int BSCannyItem::readCannyLines ()
{
  double val[5];
  int i = 0, nb = 0;
  ifstream input ("/home/even/cannyLinesModif/cannylines.txt", ios::in);
  bool reading = true;
  if (input)
  {
    while (reading)
    {
      input >> val[i];
      if (input.eof ()) reading = false;
      if (++i == 5)
      {
        CannyLine cl;
        cl.xs = (int) (val[0] + 0.5);
        cl.ys = h - 1 - (int) (val[1] + 0.5);
        cl.xe = (int) (val[2] + 0.5);
        cl.ye = h - 1 - (int) (val[3] + 0.5);
        cl.id = (int) (val[4] + 0.5);
        cannys.push_back (cl);
        i = 0;
        nb++;
      }
    }
  }
  
return (nb);
}


void BSCannyItem::drawBS (QPainter *painter)
{
  painter->setPen (QPen (BS_COLOR, DEFAULT_PEN_WIDTH, Qt::SolidLine,
                         Qt::RoundCap, Qt::RoundJoin));
  for (int i = 0; i < w * h; i++) bsmap[i] = false;
  vector<BlurredSegment *> bss = det->getBlurredSegments ();
  if (! bss.empty ())
  {
    vector<BlurredSegment *>::const_iterator it = bss.begin ();
    while (it != bss.end ())
    {
      if (*it != NULL) drawPoints (painter, (*it)->getAllPoints ());
      it++;
    }
  }
}


void BSCannyItem::drawDSS (QPainter *painter)
{
  painter->setPen (QPen (BS_COLOR, DEFAULT_PEN_WIDTH, Qt::SolidLine,
                         Qt::RoundCap, Qt::RoundJoin));
  for (int i = 0; i < w * h; i++) bsmap[i] = false;
  vector<BlurredSegment *> bss = det->getBlurredSegments ();
  if (! bss.empty ())
  {
    vector<BlurredSegment *>::const_iterator it = bss.begin ();
    while (it != bss.end ())
    {
      if ((*it) != NULL)
      {
        DigitalStraightSegment *dss = (*it)->getSegment ();
        if (dss != NULL)
        {
          vector<Pt2i> pts;
          dss->getPoints (pts);
          drawPoints (painter, pts);
        }  
      }
      it++;
    }
  }
}


void BSCannyItem::drawPoints (QPainter *painter, vector<Pt2i> pts)
{
  vector<Pt2i>::iterator iter = pts.begin ();
  while (iter != pts.end ())
  {
    Pt2i p = *iter++;
    if (p.x() < w && p.y() < h && p.x() >= 0 && p.y() >= 0)
    {
      bsmap[p.y()*w+p.x()] = true;
      painter->drawPoint (QPoint (p.x(), h - 1 - p.y()));  // dec 1
    }
  }
}


void BSCannyItem::drawCannyLines (QPainter *painter)
{
  int n = 0;
  Pt2i *pts = NULL;
  vector<CannyLine>::iterator it = cannys.begin ();
  while (it != cannys.end ())
  {
    pts = Pt2i (it->xs, it->ys).drawing (Pt2i (it->xe, it->ye), &n);
    for (int i = 0; i < n; i++)
    {
      painter->setPen (QPen (bsmap[pts[i].y()*w+pts[i].x()] ?
                             BOTH_COLOR : CANNY_COLOR,
                         DEFAULT_PEN_WIDTH, Qt::SolidLine,
                         Qt::RoundCap, Qt::RoundJoin));
      painter->drawPoint (QPoint (pts[i].x (),
                                  h - 1 - pts[i].y ()));  // dec 1
      clmap[pts[i].y()*w+pts[i].x()] = true;
    }
    delete [] pts;
    it++;
  }
}
