#ifndef BS_CANNY_ITEM_H
#define BS_CANNY_ITEM_H

#include <QGraphicsItem>
#include "bsdetector.h"

using namespace std;


/** 
 * @class BSCannyItem bscannyitem.h
 * \brief Blurred segment comparator with CannyLine.
 * \author {P. Even}
 */
class BSCannyItem : public QGraphicsItem
{
public:

  /**
   * \brief Creates a CannyLine comparator.
   */
  BSCannyItem (int width, int height, const QImage *im, BSDetector *detector);

  /**
   * \brief Deletes the CannyLine comparator.
   */
  ~BSCannyItem ();

  /**
   * \brief Returns the grid area.
   */
  QRectF boundingRect () const;

  /**
   * \brief Redraws the CannyLine comparator grid.
   */
  void paint (QPainter *painter,
              const QStyleOptionGraphicsItem *option, QWidget *widget);

  /**
   * \brief Toggles display modality.
   */
  inline void toggleType () { if (++type == 3) type = 0; }



private:

  /** Canny lines alone display modality. */
  static const int ALONE;
  /** With BS display modality. */
  static const int WITH_BS;
  /** With DSS display modality. */
  static const int WITH_DSS;
  /** Default color for blurred segments. */
  static const QColor BS_COLOR;
  /** Default color for Canny lines. */
  static const QColor CANNY_COLOR;
  /** Default color for both tools. */
  static const QColor BOTH_COLOR;
  /** Default value for pen width. */
  static const int DEFAULT_PEN_WIDTH;

  /** Aggregation of segment extraction results with initial conditions. */
  struct CannyLine
  {
    /** Canny line start point X coordinate. */
    int xs;
    /** Canny line start point Y coordinate. */
    int ys;
    /** Canny line end point X coordinate. */
    int xe;
    /** Canny line end point Y coordinate. */
    int ye;
    /** Canny line identifier. */
    int id;
  };
  /** List of Canny lines. */
  vector<CannyLine> cannys;

  /** Background image. */
  const QImage *im;
  /** Grid width. */
  int w;
  /** Grid height. */
  int h;
  /** Segment detector. */
  BSDetector *det;
  /** Blurred segments map */
  bool *bsmap;
  /** Canny lines map */
  bool *clmap;
  /** Display modality */
  int type;

  /** Reads Canny lines file */
  int readCannyLines ();

  /** Displays blurred segments points */
  void drawBS (QPainter *painter);

  /** Displays blurred segments DSS */
  void drawDSS (QPainter *painter);

  /** Displays a set of points */
  void drawPoints (QPainter *painter, vector<Pt2i> pts);

  /** Displays Canny lines */
  void drawCannyLines (QPainter *painter);

};

#endif
